#####
##
## first, some generic functions
##

# find first free uid/gid in range
# find_id passwd 100 999
find_id() {
  local db first last ids
  db=$1
  first=$2
  last=$3
  ids=$(getent $db | awk -F: "\$3 >= $first && \$3 <= $last {print \$3}")
  for i in $(seq $first $last)
  do
    if ! echo $ids |grep -q $i; then
      echo $i
      return 0
    fi
  done
  return 1
}

#
# Update uid for user from reserved system range (0-99) to dynamic system
# range (100-999).  Optionally update ownerships of given directories.
# $0 user [directory ...]
#
check_and_update_ugid() {
  local user newgid newuid
  user=$1
  if [ "$(getent passwd $user | awk -F: '$3 >= 100 {print "ok"; exit 0}')" ]; then
    return 0
  fi
  shift
  newgid=$(find_id group 100 999)
  newuid=$(find_id passwd 100 999)
  # other directories/files
  chown -R $newuid:$newgid $* 2>/dev/null || true
  groupmod -g $newgid $user
  usermod -u $newuid -g $newgid $user
  cp_echo "CN: Fixed $user user uid/gid."
}

wait_for_fds () {
  # wait until process shows some I/O readiness :)
  [ "$DEBIAN_SCRIPT_DEBUG" ] && set -vx
  local name IFSOLD num sleep maxtry script user psname pidfile fdname
  name="$1"
  [ -z "$name" ] && return 1
  IFSOLD="$IFS"
  IFS="	" # tab
  read name script user psname pidfile num fdname <<-EOPTS
	$(echo "$options" | sed 's/		*/	/g' | grep ^$name)
	EOPTS
  IFS="$IFSOLD"
  num=${num:-4}
  sleep=${sleep:-1}
  maxtry=${maxtry:-10}
  if [ -n "$pidfile" ]; then
    pidfile=/var/run/$pidfile
    findpid="[ -f $pidfile ] && cat $pidfile || true"
  else
    findpid="pgrep -u $user -f \"$psname\" -P 1 | head -1"
  fi

  # loop the loop the loop
  try=1
  while /bin/true
  do
    sleep $sleep                           # 1st, give it a chance to run
    pid=`eval $findpid`                    # 2nd: find it
    [ -z "$pid" ]           && return 1    # not running at all
    count=`ls -1 /proc/$pid/fd 2> /dev/null| wc -l`    # 3rd: count all it's worth
    [ "$count" -ge "$num" ] && ls -l /proc/$pid/fd | grep -q $fdname \
                            && return      # success -- release
    try=$(($try+1))
    [ "0$try" -ge "0$maxtry" ] && return 1 # no luck this time
  done
}

failed() {
  if [ -n "$failed" ]; then
    failed="$failed, $1"
  else
    failed="$1"
  fi
}

check_and_add_alias () {
  if ! grep -q "^$1:" $ALIASES; then
    echo "$1: $2" >> $ALIASES
    # both postfix and sendmail use newaliases
    newaliases > /dev/null
  fi
}

noisy_backup() {
  cp_backup_conffile "$1"
  cp_echo "CN: Current configuration saved in /var/backups/`basename $1`.bak"
}

# XXX TODO implement per paragraph conditional munging some day
in_paragraph() {
  return 1
}
  
commented_in_paragraph() {
  local s
  s="$1"
  shift
  s=`echo $s|sed 's/^/[[:blank:]]*#[[:blank:]]/'`
  set -- "$s" "$@"
  in_paragraph "$@"
}

uncommented_in_paragraph() {
  local s
  s="$1"
  shift
  s=`echo $s|sed 's/^/[[:blank:]]*/'`
  set -- "$s" "$@"
  in_paragraph "$@"
}

catpatch() {
  cat "$postdiff" | sed "s,amavisd\.conf[a-z.-]*\t,$1\t,"
}

#####
##
## amavisd-cn specific functions bloated enough to be moved here
##

conf_from_template() {
  cp /dev/null $ACONF
  sed -e "s/_CN_DOMAIN_/$domain/g; s/_CN_HOST_/$host/g" $ACONFTMPL >> $ACONF
  restart_daemon=1
  changed_config=1
}
